#include<fcntl.h>
#include<stdio.h>
#include<unistd.h>
#include<signal.h>
#include<stdlib.h>
#include<string.h>
#include<sys/ptrace.h>
#include<sys/types.h>
#include<sys/stat.h>
#include<sys/user.h>
#include<sys/wait.h>

#define CHILD_EXEC_PATH "./child"

void child_process();

int main(int argc, char *argv[], char *envp[])
{
	int cpid, wstatus;
	unsigned long pret;
	pid_t wpid;
	int sigpipe[2];
	char sigbuf;
	struct user_regs_struct regs;

	if(pipe(sigpipe) < 0)
	{
		perror("Unable to open pipe");
		return 1;
	}

	cpid = fork();
	if(cpid < 0)
	{
		perror("Unable to fork :");
		return 1;
	}
	else if(cpid == 0)
	{
		child_process(argv, envp, sigpipe);
	}

	printf("Child started %d\n", cpid);

	if(ptrace(PTRACE_ATTACH, cpid, 0, 0) == -1)
	{
		perror("Unable to attach to child process");
		
		goto err_cleanup;
	}

	if(waitpid(cpid, &wstatus, 0) < 0)
	{
		perror("Unable to wait for child process to stop");
		goto err_cleanup;
	}
	if(!WIFSTOPPED(wstatus))
	{
		dprintf(2, "Wstatus = %d\n", wstatus);
		goto err_cleanup;
	}


	//read(sigpipe[0], &sigbuf, 1); // sync message
	printf("Attached to %d\n", cpid);
	if(ptrace(PTRACE_SETOPTIONS, cpid, NULL, PTRACE_O_TRACEEXEC) < 0)
	{
		perror("Setoptions error");
		goto err_cleanup;
	}

	if(ptrace(PTRACE_CONT, cpid, NULL, 0) < 0)
	{
		perror("CONT error");
		goto err_cleanup;
	}

	if(waitpid(cpid, &wstatus, 0) < 0)
	{
		perror("Unable to wait for child process to stop");
		goto err_cleanup;
	}
	if(wstatus >> 8 != (SIGTRAP | (PTRACE_EVENT_EXEC<<8)))
	{
		dprintf(2, "Wstatus = %d\n", wstatus);
		goto err_cleanup;
	}

	// first syscall is execve ret
	// then mmap in & out
	for(int i=0; i<3; i++)
	{
		if(ptrace(PTRACE_SYSCALL, cpid, NULL, 0) < 0)
		{
			perror("CONT error");
			goto err_cleanup;
		}
		printf("syscall sent, waiting child to stop\n");

		if(waitpid(cpid, &wstatus, 0) < 0)
		{
			perror("Unable to wait for child process to stop");
			goto err_cleanup;
		}
		if(wstatus != 1407)
		{
			dprintf(2, "Wstatus = %d\n", wstatus);
			goto err_cleanup;
		}
	}
	printf("mmap done ?\n");


	bzero(&regs, sizeof(regs));
	if(ptrace(PTRACE_GETREGS, cpid, NULL, &regs) < 0)
	{
		perror("GETREGS error");
		goto err_cleanup;
	}

	printf("rax = %016llX rsp=%016llX r15=%016llX rip=%016llX\n",
			regs.rax, regs.rsp, regs.r15, regs.rip);

	/*
	if((pret = ptrace(PTRACE_PEEKTEXT, cpid, regs.rsp, 0)) < 0)
	{
		perror("PEEKTEXT fails");
		goto err_cleanup;
	}

	printf("mmap_addr : %08lx", pret);

	if((pret = ptrace(PTRACE_PEEKTEXT, cpid, regs.rsp+sizeof(long), 0)) < 0)
	{
		perror("PEEKTEXT fails");
		goto err_cleanup;
	}
	printf(" %08lx\n", pret);
	*/
	char mapfile[256];
	snprintf(mapfile, 256, "/proc/%d/maps", cpid);
	int mapfd = open(mapfile, O_RDONLY);
	char buff[256];
	int read_ret;

	do
	{
		read_ret = read(mapfd, buff, 255);
		buff[read_ret] = '\0';
		if(read_ret)
		{
			printf("%s", buff);
		}
	}while(read_ret > 0);

	printf("Exiting...\n");
	kill(cpid, SIGKILL);
	exit(0);

err_cleanup:
	/*
		if(waitpid(cpid, &wstatus, 0) < 0)
		{
			perror("Unable to wait for child process to stop");
			goto err_cleanup;
		}
		if(WIFEXITED(wstatus))
		{
			dprintf(2, "child exited with status %d\n", WEXITSTATUS(wstatus));

		} else if(WCOREDUMP(wstatus)) {
			dprintf(2, "child segfault\n");
		} else if(WIFSTOPPED(wstatus)) {
			dprintf(2, "child stopped\n");
		} else {
			dprintf(2, "Wstatus = %d\n", wstatus);
		}
	*/

	dprintf(2, "Hardkill %d\n", cpid);
	kill(cpid, SIGKILL);
	exit(2);

}

void child_process(char * argv[], char *envp[], int sigpipe[2])
{
	char sigbuf = '\0';
	//char* const argv[] = {"child"};
	//char* const envp[] = {};
	/*
	if(ptrace(PTRACE_TRACEME, 0,0,0) < 0)
	{
		perror("Unable to TRACEME");
		exit(5);
	}
	printf("TRACEME %d [OK]\n", getpid());
	*/
	//write(sigpipe[1], &sigbuf, 1); // sync message
	printf("%d execve\n", getpid());
	execve(CHILD_EXEC_PATH, argv, envp);
	perror("Unable to execve");
	exit(5);
}
