#ifndef ASMSH_MMAP_PARSE_H
#define ASMSH_MMAP_PARSE_H

#include "config.h"

#include <sys/mman.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <errno.h>
#include <fcntl.h>
#include <limits.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>


extern int errno;

typedef struct child_mmap_s child_mmap_t;
typedef struct child_mmap_list child_mmap_l;

struct child_mmap_s
{
	void *start;
	void *stop;
	int perm;
	size_t offset;
	dev_t device;
	ino_t inode;
	char *pathname;
};

struct child_mmap_list
{
	child_mmap_t *maps;
	size_t size;
};

/** Parse /proc/[pid]/map file
 *
 * @param pid_t The pid of the child process we ptrace attached
 * @param child_mmap_l A child map that will be filled with child's mmaps
 * @return 0 if no error else -1
 */
int child_mmap_get(pid_t child_pid, child_mmap_l *maps);

/** Parse an opened /proc/[pid]/map file
 *
 * @param int opened file descriptor
 * @param child_mmap_l A child map that will be filled with child's mmaps
 * @return 0 if no error else -1
 */
int child_mmap_get_fd(int maps_fd, child_mmap_l *maps);



int child_mmap_parseline(char *line, child_mmap_t *map);

int _child_mmap_alloc(size_t curmap, child_mmap_l *maps);

#endif
